    -- ResolverAD2S1200DecA7  - Resolver connected to AD2S1200 decoder with CmodA7 board
    -- Copyright (C) 2021 VCC
    -- release date: 31 Oct 2021
    -- author: VCC

    -- This file is part of ResolverAD2S1200DecA7 project.

    -- Permission is hereby granted, free of charge, to any person obtaining a copy
    -- of this software and associated documentation files (the "Software"),
    -- to deal in the Software without restriction, including without limitation
    -- the rights to use, copy, modify, merge, publish, distribute, sublicense,
    -- and/or sell copies of the Software, and to permit persons to whom the
    -- Software is furnished to do so, subject to the following conditions:

    -- The above copyright notice and this permission notice shall be included
    -- in all copies or substantial portions of the Software.

    -- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    -- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    -- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    -- IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
    -- DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    -- TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
    -- OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


----------------------------------------------------------------------------------
-- Company: 
-- Engineer: VCC
-- 
-- Create Date:    17:07:00 10/27/2021 
-- Design Name: 
-- Module Name:    ParallelInterface - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity ParallelInterface is
    Port (Clk100MHz: in STD_LOGIC;                            --Clock source for this component. Must be 100MHz
          
          --AD2S interface
          ResolverPosition: in STD_LOGIC_VECTOR(11 downto 0); --position from AD2S interface
          ResolverVelocity: in STD_LOGIC_VECTOR(11 downto 0); --velocity from AD2S interface
          
          --Misc
          ReadingVelocityRequest: out STD_LOGIC;              --becomes '0' when the velocity is not requested, so the AD2S interface won't even request velocity. This way, the position might be read more often. 
          
          --Parallel interface
          ParallelClk: in STD_LOGIC;                          --Clock signal from device reading from FPGA, using the parallel port. This clock is sampled by Clk100MHz, so it must be less than 50MHz.
          RDVel: in STD_LOGIC;                                --Data selector.  '0' for velocity, '1' for position
          ParallelData: out STD_LOGIC_VECTOR(11 downto 0)     --Can be position or velocity, depending on RDVel.
    );
end ParallelInterface;

architecture Behavioral of ParallelInterface is
  
  signal VelocityRequestCounter: STD_LOGIC_VECTOR(1 downto 0) := "00";
  
  signal SyncParallelClk_SReg: STD_LOGIC_VECTOR(2 downto 0); -- 3-bit shift register, to cancel metastability
  signal SyncRDVel_SReg: STD_LOGIC_VECTOR(2 downto 0); -- 3-bit shift register, to cancel metastability
  
  signal SyncParallelClk: STD_LOGIC;
  signal SyncRDVel: STD_LOGIC;
  signal OldSyncParallelClk: STD_LOGIC;
  
begin

  process (Clk100MHz)
  begin
    if Clk100MHz'event and Clk100MHz = '1' then
      SyncParallelClk_SReg <= SyncParallelClk_SReg(1 downto 0) & ParallelClk;
      SyncRDVel_SReg <= SyncRDVel_SReg(1 downto 0) & RDVel;
    end if;
  end process;
  
  
  SyncParallelClk <= SyncParallelClk_SReg(2);
  SyncRDVel <= SyncRDVel_SReg(2);

  process (Clk100MHz)
  begin
    if Clk100MHz'event and Clk100MHz = '1' then
      OldSyncParallelClk <= SyncParallelClk;
      
      if (OldSyncParallelClk = '0') and (SyncParallelClk = '1') then
        if SyncRDVel = '1' then
          ParallelData <= ResolverPosition;
          
          VelocityRequestCounter <= VelocityRequestCounter + 1;
          
          if VelocityRequestCounter = 2 then
            VelocityRequestCounter <= "00";
            ReadingVelocityRequest <= '0';
          end if;
        else
          ParallelData <= ResolverVelocity;
          
          VelocityRequestCounter <= "00";
          ReadingVelocityRequest <= '1';
        end if;
      end if;
    end if;
  end process;
   
end Behavioral;

