{   ResolverAD2S1200DecA7  - Resolver connected to AD2S1200 decoder with CmodA7 board
    Copyright (C) 2021 VCC
    release date: 31 Oct 2021
    author: VCC

    This file is part of ResolverAD2S1200DecA7 project.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included
    in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
    DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
    OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
}

unit DynTFTHandlers;

{$IFNDEF IsMCU}
  {$DEFINE IsDesktop}
{$ENDIF}

{$IFDEF IsDesktop}
interface
{$ENDIF}

uses
  {$IFDEF UseSmallMM}
    DynTFTSmallMM,
  {$ELSE}
    {$IFDEF IsDesktop}
      MemManager,
    {$ENDIF}  
  {$ENDIF} //this must be the first unit, at least in Delphi, because it exports GetMem.

  DynTFTTypes, DynTFTConsts, DynTFTUtils, DynTFTBaseDrawing, DynTFTControls,
  DynTFTGUIObjects,


//<DynTFTComponents>
  DynTFTButton,
  DynTFTArrowButton,
  //DynTFTPanel,
  //DynTFTCheckBox,
  DynTFTScrollBar,
  DynTFTItems,
  DynTFTListBox,
  DynTFTLabel,
  //DynTFTRadioButton,
  //DynTFTRadioGroup,
  DynTFTTabButton,
  DynTFTPageControl,
  DynTFTEdit,
  //DynTFTKeyButton,
  //DynTFTVirtualKeyboard,
  DynTFTComboBox,
  DynTFTTrackBar,
  //DynTFTProgressBar,
  DynTFTMessageBox
//<EndOfDynTFTComponents> - Do not remove or modify this line!

  {$IFDEF IsDesktop}
    ,SysUtils, Forms
    {$IFDEF DynTFTFontSupport}, DynTFTFonts {$ENDIF}
  {$ENDIF}

  {$IFNDEF UserTFTCommands}
    {$IFDEF IsDesktop} , TFT {$ENDIF}
  {$ELSE}
    , {$I UserDrawingUnits.inc}
  {$ENDIF}

  {$I DynTFTHandlersAdditionalUnits.inc}
  ;
  
  {$IFDEF UseHandlersAdditionalCodeInInterface}
    {$I DynTFTHandlersAdditionalCodeInInterface.inc} //Can be used to specify external functions, vars, consts
  {$ENDIF}

//CodegenSym:GroupsBegin
//CodegenSym:GroupsEnd

procedure PageControl1_OnChange(Sender: PPtrRec); //CodegenSym:header
procedure trbBackLight_OnTrackBarChange(Sender: PPtrRec); //CodegenSym:header
procedure scrbarAmplification_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:header
procedure scrbarTriggerLevel_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:header
procedure btnResetAD2S_OnMouseUpUser(Sender: PPtrRec); //CodegenSym:header
procedure cmbExcitationFrequency_OnComboBoxCloseUp(Sender: PPtrRec); //CodegenSym:header
procedure scrbarTiming_SampleOn0_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:header
procedure scrbarTiming_RDCS0_ForPos_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:header
procedure scrbarTiming_ReadPos_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:header
procedure scrbarTiming_RDVELOutOn0_ForVel_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:header
procedure scrbarTiming_RDCS0_ForVel_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:header
procedure scrbarTiming_ReadVel_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:header
procedure scrbarTiming_EndCycle_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:header
procedure btnResetAllTimings_OnMouseUpUser(Sender: PPtrRec); //CodegenSym:header
procedure btnLoadSettingsFromFlash_OnMouseUpUser(Sender: PPtrRec); //CodegenSym:header
procedure btnSaveSettingsToFlash_OnMouseUpUser(Sender: PPtrRec); //CodegenSym:header

//CodegenSym:AllBinHandlersBegin

{$IFDEF RTTIREG}

  procedure ExecuteDesignRTTIInstructions_Create(Filter: Byte);   //If a compiler complains about missing implementation, probably regenerating the output files from DynTFTCodeGen, will fix the issue.
  procedure ExecuteDesignRTTIInstructions_Destroy(Filter: Byte);  //If a compiler complains about missing implementation, probably regenerating the output files from DynTFTCodeGen, will fix the issue.

  {$IFDEF IsDesktop} {$IFDEF DesktopApp_FP} // Profile: "DesktopApp_FP"
      var
        AllBinHandlers: array[0..0] of TPtr;

      procedure UpdateAllBinHandlerAddressArray;

    procedure LinkHandlers;
  {$ENDIF} {$ENDIF} // IsDesktop

  {$IFDEF IsMCU} {$IFDEF PIC32App} // Profile: "PIC32App"
      var
        AllBinHandlers: array[0..0] of TPtr;

      procedure UpdateAllBinHandlerAddressArray;

    procedure LinkHandlers;
  {$ENDIF} {$ENDIF} // IsMCU

  {$IFDEF IsDesktop}
      var
        AllBinHandlersStr: array[0..0] of string;
        AllBinHandlersAddresses: array[0..0] of TPtr;

        AllBinIdentifiersStr: array[0..0] of string;  // No identifiers found. Using a dummy entry.
        AllBinIdentifiersAddresses: array[0..0] of TPtr;  // No identifiers found. Using a dummy entry.

      procedure UpdateAllBinHandlerStrArray;
  {$ENDIF}
{$ENDIF} // RTTIREG

//CodegenSym:AllBinHandlersEnd

implementation

{$I DynTFTHandlersAdditionalCode.inc}

//CodegenSym:UpdateBinHandlersProcBegin
{$IFDEF RTTIREG}
  {$IFDEF IsDesktop} {$IFDEF DesktopApp_FP} // Profile: "DesktopApp_FP"
      procedure UpdateAllBinHandlerAddressArray;
      begin
        AllBinHandlers[0] := TPtr(@trbBackLight_OnTrackBarChange);
      end;

    procedure ExecuteDesignRTTIInstructions_Create(Filter: Byte);
    var
      GetDataCallback: {$IFDEF IsDesktop} TDynTFTExecRTTIInstructionCallback {$ELSE} PDynTFTExecRTTIInstructionCallback {$ENDIF};
    begin
      GetDataCallback := nil;  // no data provider callback is used. ExecuteRTTIInstructions expects that "C_RTTI_CreateInstructionData" holds all instructions.
      ExecuteRTTIInstructions(@C_RTTI_CreateInstructionData, @CAllCreatedBinComponents, @AllBinHandlers, Filter, GetDataCallback);
    end;

    procedure ExecuteDesignRTTIInstructions_Destroy(Filter: Byte);
    var
      GetDataCallback: {$IFDEF IsDesktop} TDynTFTExecRTTIInstructionCallback {$ELSE} PDynTFTExecRTTIInstructionCallback {$ENDIF};
    begin
      GetDataCallback := nil;  // no data provider callback is used. ExecuteRTTIInstructions expects that "C_RTTI_DestroyInstructionData" holds all instructions.
      ExecuteRTTIInstructions(@C_RTTI_DestroyInstructionData, @CAllCreatedBinComponents, @AllBinHandlers, Filter, GetDataCallback);
    end;

    procedure LinkHandlers;
    {$IFDEF IsMCU}
      var
        Dummy: DWord; volatile;
    {$ENDIF}
    begin
      SetFuncCall(trbBackLight_OnTrackBarChange);
    end;
  {$ENDIF} {$ENDIF} // IsDesktop {$IFDEF DesktopApp_FP}

  {$IFDEF IsMCU} {$IFDEF PIC32App} // Profile: "PIC32App"
      procedure UpdateAllBinHandlerAddressArray;
      begin
        AllBinHandlers[0] := TPtr(@trbBackLight_OnTrackBarChange);
      end;

    procedure ExecuteDesignRTTIInstructions_Create(Filter: Byte);
    var
      GetDataCallback: {$IFDEF IsDesktop} TDynTFTExecRTTIInstructionCallback {$ELSE} PDynTFTExecRTTIInstructionCallback {$ENDIF};
    begin
      GetDataCallback := nil;  // no data provider callback is used. ExecuteRTTIInstructions expects that "C_RTTI_CreateInstructionData" holds all instructions.
      ExecuteRTTIInstructions(@C_RTTI_CreateInstructionData, @CAllCreatedBinComponents, @AllBinHandlers, Filter, GetDataCallback);
    end;

    procedure ExecuteDesignRTTIInstructions_Destroy(Filter: Byte);
    var
      GetDataCallback: {$IFDEF IsDesktop} TDynTFTExecRTTIInstructionCallback {$ELSE} PDynTFTExecRTTIInstructionCallback {$ENDIF};
    begin
      GetDataCallback := nil;  // no data provider callback is used. ExecuteRTTIInstructions expects that "C_RTTI_DestroyInstructionData" holds all instructions.
      ExecuteRTTIInstructions(@C_RTTI_DestroyInstructionData, @CAllCreatedBinComponents, @AllBinHandlers, Filter, GetDataCallback);
    end;

    procedure LinkHandlers;
    {$IFDEF IsMCU}
      var
        Dummy: DWord; volatile;
    {$ENDIF}
    begin
      SetFuncCall(trbBackLight_OnTrackBarChange);
    end;
  {$ENDIF} {$ENDIF} // IsMCU {$IFDEF PIC32App}


  {$IFDEF IsDesktop}
      procedure UpdateAllBinHandlerStrArray;
      begin
        AllBinHandlersStr[0] := '_trbBackLight_OnTrackBarChange';

        AllBinHandlersAddresses[0] := TPtr(@trbBackLight_OnTrackBarChange);

        AllBinIdentifiersStr[0] := 'DummyProc';
        AllBinIdentifiersAddresses[0] := TPtr(nil);
      end;
  {$ENDIF}
{$ENDIF} // RTTIREG

//CodegenSym:UpdateBinHandlersProcEnd

//CodegenSym:CreationGroups

//CodegenSym:HandlersImplementation

procedure PageControl1_OnChange(Sender: PPtrRec); //CodegenSym:handler
var
  i: Integer;
begin //CodegenSym:handler:begin
  {$IFDEF IsDesktop}
    DynTFT_DebugConsole('--------------------------- changing tab to ' + IntToStr(PageControl1^.ActiveIndex + 1));
  {$ENDIF}

  DynTFT_Set_Pen(DynTFTAllComponentsContainer[PageControl1^.ActiveIndex + 1].ScreenColor, 1);
  DynTFT_Set_Brush(1, DynTFTAllComponentsContainer[PageControl1^.ActiveIndex + 1].ScreenColor, 0, 0, 0, 0);
  DynTFT_Rectangle(0, 28, TFT_DISP_WIDTH, TFT_DISP_HEIGHT);

  for i := 1 to PageControl1^.PageCount {- 1} do
  begin
    DynTFTAllComponentsContainer[i].Active := PageControl1^.ActiveIndex = i - 1;
    if DynTFTAllComponentsContainer[i].Active then
      DynTFTRepaintScreenComponents(i, CREPAINTONSTARTUP, nil);
  end;

  {$IFDEF IsMCU}
    UpdateOpAMPGainToUI;
  
    if DynTFTAllComponentsContainer[lstStatus^.BaseProps.ScreenIndex].Active then
      PaintResStatusSignals(True);
  {$ENDIF}
end; //CodegenSym:handler:end


procedure trbBackLight_OnTrackBarChange(Sender: PPtrRec); //CodegenSym:handler
begin //CodegenSym:handler:begin
  {$IFDEF IsMCU}
    {$IFNDEF UserTFTCommands}
      TFT_Set_DBC_SSD1963(trbBackLight^.Position);
    {$ENDIF}
  {$ENDIF}
end; //CodegenSym:handler:end


procedure scrbarAmplification_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:handler
begin //CodegenSym:handler:begin
  {$IFDEF IsMCU}
    UpdateOpAMPGainToUI;
  {$ENDIF}
end; //CodegenSym:handler:end


procedure scrbarTriggerLevel_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:handler
var
  TriggerLevelStr: string{$IFDEF IsMCU}[6]{$ENDIF};
  {$IFDEF IsDesktop}
    TriggerLevel: Integer; //this is an external variable in the PIC application, but is not used on desktop simulator
  {$ENDIF}
begin //CodegenSym:handler:begin
  TriggerLevel := scrbarTriggerLevel^.Position;
  {$IFDEF IsMCU}
    Int2Str(TriggerLevel, TriggerLevelStr);
  {$ELSE}
    TriggerLevelStr := IntToStr(TriggerLevel);
  {$ENDIF}
  
  DynTFTLabel_UpdateCaption(lblTriggerLevelValue, TriggerLevelStr);
end; //CodegenSym:handler:end


procedure btnResetAD2S_OnMouseUpUser(Sender: PPtrRec); //CodegenSym:handler
var
  MBMsg: string{$IFDEF IsMCU}[32]{$ENDIF};
  MBTitle: string{$IFDEF IsMCU}[11]{$ENDIF};
begin //CodegenSym:handler:begin
  MBMsg := 'Are you sure you want to reset?';
  MBTitle := 'AD2S Reset';

  {$IFDEF IsMCU}
    EnableVelocityDisplay := 0;
  {$ENDIF}

  if DynTFTShowMessageBox(btnResetAD2S^.BaseProps.ScreenIndex, MBMsg, MBTitle, CDynTFT_MB_YESNO) = CDynTFT_IDYES then
  begin
    {$IFDEF IsMCU}
      SendAD2SResetCommand;
    {$ENDIF}
  end;

  {$IFDEF IsMCU}
    EnableVelocityDisplay := 1;
  {$ENDIF}
end; //CodegenSym:handler:end


procedure cmbExcitationFrequency_OnComboBoxCloseUp(Sender: PPtrRec); //CodegenSym:handler
begin //CodegenSym:handler:begin
  {$IFDEF IsMCU}
    SendAD2SSettingsCommand(cmbExcitationFrequency^.ListBox^.Items^.ItemIndex);
  {$ENDIF}
end; //CodegenSym:handler:end


procedure scrbarTiming_SampleOn0_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:handler
begin //CodegenSym:handler:begin
  {$IFDEF IsMCU}
    UpdateAD2STimingParameter(scrbarTiming_SampleOn0, lblTiming_SampleOn0_Value, CTiming_SampleOn0);
  {$ENDIF}
end; //CodegenSym:handler:end


procedure scrbarTiming_RDCS0_ForPos_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:handler
begin //CodegenSym:handler:begin
  {$IFDEF IsMCU}
    UpdateAD2STimingParameter(scrbarTiming_RDCS0_ForPos, lblTiming_RDCS0_ForPos_Value, CTiming_RDCS0_ForPos);
  {$ENDIF}
end; //CodegenSym:handler:end


procedure scrbarTiming_ReadPos_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:handler
begin //CodegenSym:handler:begin
  {$IFDEF IsMCU}
    UpdateAD2STimingParameter(scrbarTiming_ReadPos, lblTiming_ReadPos_Value, CTiming_ReadPos);
  {$ENDIF}
end; //CodegenSym:handler:end


procedure scrbarTiming_RDVELOutOn0_ForVel_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:handler
begin //CodegenSym:handler:begin
  {$IFDEF IsMCU}
    UpdateAD2STimingParameter(scrbarTiming_RDVELOutOn0_ForVel, lblTiming_RDVELOutOn0_ForVel_Value, CTiming_RDVELOutOn0_ForVel);
  {$ENDIF}
end; //CodegenSym:handler:end


procedure scrbarTiming_RDCS0_ForVel_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:handler
begin //CodegenSym:handler:begin
  {$IFDEF IsMCU}
    UpdateAD2STimingParameter(scrbarTiming_RDCS0_ForVel, lblTiming_RDCS0_ForVel_Value, CTiming_RDCS0_ForVel);
  {$ENDIF}
end; //CodegenSym:handler:end


procedure scrbarTiming_ReadVel_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:handler
begin //CodegenSym:handler:begin
  {$IFDEF IsMCU}
    UpdateAD2STimingParameter(scrbarTiming_ReadVel, lblTiming_ReadVel_Value, CTiming_ReadVel);
  {$ENDIF}
end; //CodegenSym:handler:end


procedure scrbarTiming_EndCycle_OnScrollBarChange(Sender: PPtrRec); //CodegenSym:handler
begin //CodegenSym:handler:begin
  {$IFDEF IsMCU}
    UpdateAD2STimingParameter(scrbarTiming_EndCycle, lblTiming_EndCycle_Value, CTiming_EndCycle);
  {$ENDIF}
end; //CodegenSym:handler:end


procedure btnResetAllTimings_OnMouseUpUser(Sender: PPtrRec); //CodegenSym:handler
begin //CodegenSym:handler:begin
  {$IFDEF IsMCU}
    scrbarTiming_SampleOn0^.Position := CTiming_SampleOn0_DefaultValue;
    scrbarTiming_RDCS0_ForPos^.Position := CTiming_RDCS0_ForPos_DefaultValue;
    scrbarTiming_ReadPos^.Position := CTiming_ReadPos_DefaultValue;
    scrbarTiming_RDVELOutOn0_ForVel^.Position := CTiming_RDVELOutOn0_ForVel_DefaultValue;
    scrbarTiming_RDCS0_ForVel^.Position := CTiming_RDCS0_ForVel_DefaultValue;
    scrbarTiming_ReadVel^.Position := CTiming_ReadVel_DefaultValue;
    scrbarTiming_EndCycle^.Position := CTiming_EndCycle_DefaultValue;
    
    DynTFTDrawScrollBar(scrbarTiming_SampleOn0, True);
    DynTFTDrawScrollBar(scrbarTiming_RDCS0_ForPos, True);
    DynTFTDrawScrollBar(scrbarTiming_ReadPos, True);
    DynTFTDrawScrollBar(scrbarTiming_RDVELOutOn0_ForVel, True);
    DynTFTDrawScrollBar(scrbarTiming_RDCS0_ForVel, True);
    DynTFTDrawScrollBar(scrbarTiming_ReadVel, True);
    DynTFTDrawScrollBar(scrbarTiming_EndCycle, True);
    
    UpdateAD2STimingParameter(scrbarTiming_SampleOn0, lblTiming_SampleOn0_Value, CTiming_SampleOn0);
    UpdateAD2STimingParameter(scrbarTiming_RDCS0_ForPos, lblTiming_RDCS0_ForPos_Value, CTiming_RDCS0_ForPos);
    UpdateAD2STimingParameter(scrbarTiming_ReadPos, lblTiming_ReadPos_Value, CTiming_ReadPos);
    UpdateAD2STimingParameter(scrbarTiming_RDVELOutOn0_ForVel, lblTiming_RDVELOutOn0_ForVel_Value, CTiming_RDVELOutOn0_ForVel);
    UpdateAD2STimingParameter(scrbarTiming_RDCS0_ForVel, lblTiming_RDCS0_ForVel_Value, CTiming_RDCS0_ForVel);
    UpdateAD2STimingParameter(scrbarTiming_ReadVel, lblTiming_ReadVel_Value, CTiming_ReadVel);
    UpdateAD2STimingParameter(scrbarTiming_EndCycle, lblTiming_EndCycle_Value, CTiming_EndCycle);
  {$ENDIF}
end; //CodegenSym:handler:end


procedure btnLoadSettingsFromFlash_OnMouseUpUser(Sender: PPtrRec); //CodegenSym:handler
var
  MBMsg: string{$IFDEF IsMCU}[32]{$ENDIF};
  MBTitle: string{$IFDEF IsMCU}[11]{$ENDIF};
begin //CodegenSym:handler:begin
  MBMsg := 'Continue loading from flash?';
  MBTitle := 'Loading';

  if DynTFTShowMessageBox(btnLoadSettingsFromFlash^.BaseProps.ScreenIndex, MBMsg, MBTitle, CDynTFT_MB_YESNO) = CDynTFT_IDYES then
  begin
    {$IFDEF IsMCU}
      LoadSettingsFromFlash;
     {$ENDIF}
    
    MBMsg := 'Done';
    DynTFTShowMessageBox(btnLoadSettingsFromFlash^.BaseProps.ScreenIndex, MBMsg, MBTitle, CDynTFT_MB_OK);
  end;
end; //CodegenSym:handler:end


procedure btnSaveSettingsToFlash_OnMouseUpUser(Sender: PPtrRec); //CodegenSym:handler
var
  MBMsg: string{$IFDEF IsMCU}[32]{$ENDIF};
  MBTitle: string{$IFDEF IsMCU}[11]{$ENDIF};
begin //CodegenSym:handler:begin
  MBMsg := 'Continue saving to flash?';
  MBTitle := 'Saving';

  if DynTFTShowMessageBox(btnLoadSettingsFromFlash^.BaseProps.ScreenIndex, MBMsg, MBTitle, CDynTFT_MB_YESNO) = CDynTFT_IDYES then
  begin
    {$IFDEF IsMCU}
      SaveSettingsToFlash;
    {$ENDIF}
    
    MBMsg := 'Done';
    DynTFTShowMessageBox(btnLoadSettingsFromFlash^.BaseProps.ScreenIndex, MBMsg, MBTitle, CDynTFT_MB_OK);
  end;
end; //CodegenSym:handler:end


end.